package com.ldodds.foaf.jena;

import java.util.*;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import com.ldodds.foaf.FOAFGraph;
import com.ldodds.foaf.model.Person;

/**
 * @author ldodds
 *  
 */
public class FOAFGraphImpl implements FOAFGraph
{
    private Model _model;

    public FOAFGraphImpl(Model model)
    {
        _model = model;
    }

    /** 
     * @see com.ldodds.foaf.FOAFGraph#smush()
     */
    public void smush()
    {
        //TODO implement smushing
    }
    
    /** 
     * @see com.ldodds.foaf.FOAFGraph#findAllPeople()
     */
    public List findAllPeople()
    {
        return findPeopleUsingSelector( new SimpleSelector(null, RDF.type, FOAF.Person) );
    }

    private List getFriends(Resource person)
    {
        List friends = new ArrayList();
        for ( StmtIterator iterator = _model.listStatements(ModelUtils
                .getKnowsSelector(person)); iterator.hasNext(); )
        {
            Statement statement = (Statement) iterator.next();
            Resource resource = (Resource) statement.getObject();
            friends.add(new PersonBuilder(resource, new Person()).build());
        }
        return friends;
    }

    /**
     * @see FOAFGraph#findPrimaryPerson(String)
     */
    public Person findPrimaryPerson(String base)
    {
        Resource primary = PrimaryPersonSniffer.findPrimaryResource(_model,
                base, false);
        if ( primary == null )
        {
            return null;
        }

        Person p = new PersonBuilder(primary, new Person()).build();
        p.setFriends(getFriends(primary));
        return p;
    }

    /**
     * @see FOAFGraph#findPrimaryPerson()
     */
    public Person findPrimaryPerson()
    {
        return findPrimaryPerson("");
    }

    /** 
     * @see com.ldodds.foaf.FOAFGraph#findPersonWithProperty(java.lang.String)
     */
    public List findPersonWithProperty(String propertyURI)
    {
        Property property = _model.createProperty(propertyURI);
        Selector selector = new SimpleSelector(null, property, (RDFNode)null);
        return findPeopleUsingSelector(selector);
    }
    /** 
     * @see com.ldodds.foaf.FOAFGraph#findPersonByProperty(java.lang.String, java.lang.String)
     */
    public List findPersonByProperty(String propertyURI, String value)
    {
        Property property = _model.createProperty(propertyURI);
        Selector selector = new ExtendedSelector(property, value);
        return findPeopleUsingSelector(selector);
    }

    private List findPeopleUsingSelector(Selector selector)
    {
        List people = new ArrayList();
        for (StmtIterator iterator = _model.listStatements(selector); iterator.hasNext(); )
        {
	       Statement statement = (Statement)iterator.next();
	       Resource subject = (Resource)statement.getSubject();
	       Person person = new Person();
	       people.add( new PersonBuilder(subject, person).build() );
	       person.setFriends( getFriends(subject) );
        }
        return people;
        
    }
}