package com.ldodds.foaf.jena;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;

/**
 * Utility class for working with <code>foaf:Person</code> resources within a
 * Jena model.
 * 
 * @author ldodds
 */
public class ModelUtils
{

    public static Selector getKnowsSelector(final Resource person)
    {
        return new Selector() {
            public boolean test(Statement statement)
            {
                if ( statement.getSubject().equals(person)
                        && statement.getPredicate().getLocalName().equals(
                                "knows") )
                {
                    return true;
                }
                return false;
            }

            public Resource getSubject()
            {
                return null;
            }

            public Property getPredicate()
            {
                return null;
            }

            public RDFNode getObject()
            {
                return null;
            }

            public boolean isSimple()
            {
                return false;
            }
        };
    }

    public static Resource findTopicOfPersonalProfileDocument(Model model,
            String base)
    {
        Resource ppd = model.getResource(base);

        Resource primaryTopic = null;
        if ( ppd.hasProperty(FOAF.primaryTopic) )
        {
            primaryTopic = (Resource) ppd.getProperty(FOAF.primaryTopic)
                    .getObject();
        }
        return primaryTopic;
    }

    public static Resource findMaker(Model model, String base)
    {
        Selector makerSelector = new SimpleSelector(model.createResource(base),
                FOAF.maker, (Object) null);

        StmtIterator iterator = model.listStatements(makerSelector);
        if ( !iterator.hasNext() )
        {
            return null;
        }
        Statement statement = (Statement) iterator.next();
        return (Resource) statement.getObject();
    }

    public static Resource findCreator(Model model, String base)
    {
        Selector creatorSelector = new SimpleSelector(model
                .createResource(base), DC.creator, (Object) null);

        StmtIterator iterator = model.listStatements(creatorSelector);
        if ( !iterator.hasNext() )
        {
            return null;
        }
        Statement statement = (Statement) iterator.next();
        return (Resource) statement.getObject();
    }

    public static boolean isFoafPerson(Resource resource)
    {
        return resource.hasProperty(RDF.type, FOAF.Person);
    }
}