package com.ldodds.foaf.jena;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import com.ldodds.foaf.model.*;

/**
 * Populates a <code>Person</code> bean instance from the properties
 * associated with a <code>foaf:Person</code> resource.
 * 
 * FIXME -- can't handle multiple properties of same type, e.g. nick, blog, seeAlso...
 * 
 * @author ldodds
 */
public class PersonBuilder
{
    private Resource _resource;

    private Person _person;

    public PersonBuilder(Resource resource, Person person)
    {
        _resource = resource;
        _person = person;
    }

    public Person build()
    {
        //Agent properties        
        String email = getPropertyWithoutPrefix(":", FOAF.email);
        _person.setEmail(email);
        _person.setEmailSha1Sum(getProperty(FOAF.email_encrypted));
        _person.setName(getProperty(FOAF.name));
        _person.setHomepage(getProperty(FOAF.homepage));
        _person.setNick(getProperty(FOAF.nickname));
        _person.setWeblog(getProperty(FOAF.weblog));
        _person.setAimChatId(getProperty(FOAF.aim));
        _person.setIcqChatId(getProperty(FOAF.icq));
        _person.setJabberId(getProperty(FOAF.jabber));
        _person.setMsnChatId(getProperty(FOAF.msn));
        _person.setYahooChatId(getProperty(FOAF.yahoo));
        _person.setGender(getProperty(FOAF.gender));
        //TODO fetch Groups
        //TODO tipjar
        
        //Person properties
        _person.setDepiction(getProperty(FOAF.depiction));        
        _person.setFirstName(getProperty(FOAF.firstname));
        _person.setSurname(getProperty(FOAF.lastname));
        _person.setSchool(getProperty(FOAF.school));
        _person.setTelephoneNumber(getPropertyWithoutPrefix(":", FOAF.phone));
        _person.setTitle(getProperty(FOAF.title));
        _person.setWorkHomepage(getProperty(FOAF.work));
        _person.setWorkInfoPage(getProperty(FOAF.about_work));

        if ( _resource.hasProperty(RDFS.seeAlso) )
        {
            _person.setSeeAlso(_resource.getProperty(RDFS.seeAlso).getObject()
                    .toString());
        }

        return _person;
    }

    private String getProperty(Property property)
    {
        if ( _resource.hasProperty(property) )
        {
            return _resource.getProperty(property).getObject().toString();
        }
        return null;
    }

    private String getPropertyWithoutPrefix(String separator, Property property)
    {
        String p = getProperty(property);
        if ( p == null )
        {
            return null;
        }
        return p.substring(p.indexOf(separator) + 1);
    }

}